from ..factory import list_algo, get_algo
from ..utils import config_list_to_dict, yprint


def show_algo(args):
    if args.algo_name is None:
        yprint(list_algo())
        return

    algo, configs = get_algo(args.algo_name)
    if algo is None:
        return
    yprint(configs)



from pkg_resources import get_distribution
from PyQt5.QtWidgets import QMainWindow, QStackedWidget, QDesktopWidget
from ..pages.home_page import BadgerHomePage
from ..pages.routine_page import BadgerRoutinePage


class BadgerMainWindow(QMainWindow):
    def __init__(self):
        super().__init__()

        self.init_ui()
        self.config_logic()

    def init_ui(self):
        version = get_distribution('badger-opt').version
        self.setWindowTitle(f'Badger v{version}')
        # self.setWindowIcon(QIcon('badger.png'))
        self.resize(864, 648)
        self.center()

        # Add pages
        self.home_page = BadgerHomePage(self.show_routine_page)
        self.routine_page = BadgerRoutinePage(self.show_home_page)

        self.stacks = stacks = QStackedWidget()
        stacks.addWidget(self.home_page)
        stacks.addWidget(self.routine_page)

        stacks.setCurrentIndex(0)

        self.setCentralWidget(self.stacks)

    def center(self):
        qr = self.frameGeometry()
        cp = QDesktopWidget().availableGeometry().center()

        qr.moveCenter(cp)
        self.move(qr.topLeft())

    def config_logic(self):
        pass

    def show_routine_page(self, routine=None):
        self.routine_page.refresh_ui(routine)
        self.stacks.setCurrentIndex(1)

    def show_home_page(self):
        self.home_page.refresh_ui()
        self.home_page.reconfig_logic()
        self.stacks.setCurrentIndex(0)



from PyQt5.QtWidgets import QHBoxLayout, QWidget, QVBoxLayout, QCheckBox, QDoubleSpinBox
from .labeled_spinbox import labeled_spinbox


def variable_item(name, vrange, callback=None):
    widget = QWidget()
    vbox = QVBoxLayout(widget)
    vbox.setSpacing(0)
    widget.check_name = check_name = QCheckBox(name)
    check_name.setChecked(True)
    vbox.addWidget(check_name)
    widget_range = QWidget()
    hbox = QHBoxLayout(widget_range)
    hbox.setContentsMargins(0, 0, 0, 0)
    widget.sb_lower = sb_lower = labeled_spinbox('min', vrange[0])
    widget.sb_upper = sb_upper = labeled_spinbox('max', vrange[1])
    hbox.addWidget(sb_lower, 1)
    hbox.addSpacing(16)
    hbox.addWidget(sb_upper, 1)
    vbox.addWidget(widget_range)

    def toggle_item():
        widget_range.setDisabled(not check_name.isChecked())
        if callback is not None:
            callback(name)

    check_name.stateChanged.connect(toggle_item)

    return widget



import argparse

from .actions import show_info
from .actions.doctor import self_check
from .actions.routine import show_routine
from .actions.algo import show_algo
from .actions.env import show_env
from .actions.intf import show_intf
from .actions.run import run_routine
from .actions.test import run_test


def main():
    # Create the top-level parser
    parser = argparse.ArgumentParser(description='Badger the optimizer')
    parser.add_argument('-g', '--gui', action='store_true',
                        help='launch the GUI')
    parser.set_defaults(func=show_info)
    subparsers = parser.add_subparsers(help='Badger commands help')

    # Parser for the 'doctor' command
    parser_doctor = subparsers.add_parser(
        'doctor', help='Badger status self-check')
    parser_doctor.set_defaults(func=self_check)

    # Parser for the 'routine' command
    parser_routine = subparsers.add_parser('routine', help='Badger routines')
    parser_routine.add_argument('routine_name',
                                nargs='?', type=str, default=None)
    parser_routine.add_argument('-r', '--run', action='store_true',
                                help='run the routine')
    parser_routine.add_argument('-y', '--yes', action='store_true',
                                help='run the routine without confirmation')
    parser_routine.add_argument('-v', '--verbose', type=int, choices=[0, 1, 2],
                                default=2, const=2, nargs='?',
                                help='verbose level of optimization progress')
    parser_routine.set_defaults(func=show_routine)

    # Parser for the 'algo' command
    parser_algo = subparsers.add_parser('algo', help='Badger algorithms')
    parser_algo.add_argument('algo_name', nargs='?', type=str, default=None)
    parser_algo.set_defaults(func=show_algo)

    # Parser for the 'intf' command
    parser_intf = subparsers.add_parser('intf', help='Badger interfaces')
    parser_intf.add_argument('intf_name', nargs='?', type=str, default=None)
    parser_intf.set_defaults(func=show_intf)

    # Parser for the 'env' command
    parser_env = subparsers.add_parser('env', help='Badger environments')
    parser_env.add_argument('env_name', nargs='?', type=str, default=None)
    parser_env.set_defaults(func=show_env)

    # Parser for the 'run' command
    parser_run = subparsers.add_parser('run', help='run routines')
    parser_run.add_argument('-a', '--algo', required=True,
                            help='algorithm to use')
    parser_run.add_argument('-ap', '--algo_params',
                            help='parameters for the algorithm')
    parser_run.add_argument('-e', '--env', required=True,
                            help='environment to use')
    parser_run.add_argument('-ep', '--env_params',
                            help='parameters for the environment')
    parser_run.add_argument('-c', '--config', required=True,
                            help='config for the routine')
    parser_run.add_argument('-s', '--save', nargs='?', const='',
                            help='the routine name to be saved')
    parser_run.add_argument('-y', '--yes', action='store_true',
                            help='run the routine without confirmation')
    parser_run.add_argument('-v', '--verbose', type=int, choices=[0, 1, 2],
                            default=2, const=2, nargs='?',
                            help='verbose level of optimization progress')
    parser_run.set_defaults(func=run_routine)

    # Parser for the 'test' command
    parser_test = subparsers.add_parser('test', help='for dev')
    parser_test.add_argument('-a', '--algo', required=True,
                             help='algorithm to use')
    parser_test.add_argument('-ap', '--algo_params',
                             help='parameters for the algorithm')
    parser_test.add_argument('-e', '--env', required=True,
                             help='environment to use')
    parser_test.add_argument('-ep', '--env_params',
                             help='parameters for the environment')
    parser_test.add_argument('-c', '--config', required=True,
                             help='config for the routine')
    parser_test.add_argument('-s', '--save', default='',
                             help='the routine name to be saved')
    parser_test.set_defaults(func=run_test)

    args = parser.parse_args()
    args.func(args)


if __name__ == '__main__':
    main()



from PyQt5.QtWidgets import QLineEdit, QCompleter


def search_bar(word_list):
    completer = QCompleter(word_list)

    line_edit = QLineEdit()
    line_edit.setPlaceholderText('Search')
    line_edit.setCompleter(completer)

    return line_edit



import os
import sys
from datetime import datetime
import logging
import yaml
import sqlite3


# Check badger database root
BADGER_DB_ROOT = os.getenv('BADGER_DB_ROOT')
if BADGER_DB_ROOT is None:
    logging.error('Please set the BADGER_DB_ROOT env var!')
    sys.exit()
elif not os.path.exists(BADGER_DB_ROOT):
    os.makedirs(BADGER_DB_ROOT)
    logging.info(
        f'Badger database root {BADGER_DB_ROOT} created')


def save_routine(routine):
    db_routine = os.path.join(BADGER_DB_ROOT, 'routines.db')

    # Initialize
    if not os.path.exists(db_routine):
        con = sqlite3.connect(db_routine)
        cur = con.cursor()
        cur.execute('create table routine (name not null primary key, config, savedAt timestamp)')
    else:
        con = sqlite3.connect(db_routine)
        cur = con.cursor()

    # Insert a record
    try:
        cur.execute('insert into routine values (?, ?, ?)',
                    (routine['name'], yaml.dump(routine, sort_keys=False), datetime.now()))
    except sqlite3.OperationalError:
        cur.execute('create table routine (name not null primary key, config, savedAt timestamp)')
        cur.execute('insert into routine values (?, ?, ?)',
                    (routine['name'], yaml.dump(routine, sort_keys=False), datetime.now()))
    con.commit()
    con.close()


def load_routine(name):
    db_routine = os.path.join(BADGER_DB_ROOT, 'routines.db')
    con = sqlite3.connect(db_routine)
    cur = con.cursor()

    cur.execute('select * from routine where name=:name', {'name': name})
    records = cur.fetchall()
    con.close()

    if len(records) == 1:
        return yaml.safe_load(records[0][1]), records[0][2]
    elif len(records) == 0:
        logging.warn(f'Routine {name} not found in the database!')
        return None, None
    else:
        raise Exception(
            f'Multiple routines with name {name} found in the database!')


def list_routines():
    db_routine = os.path.join(BADGER_DB_ROOT, 'routines.db')
    con = sqlite3.connect(db_routine)
    cur = con.cursor()

    cur.execute('select name, savedAt from routine')
    records = cur.fetchall()
    names = [record[0] for record in records]
    timestamps = [record[1] for record in records]
    con.close()

    return names, timestamps


def save_run(run):
    pass


def load_run(name):
    pass



import numpy as np
from abc import ABC, abstractmethod
from typing import List
from .interface import Interface
from .utils import merge_params


class Environment(ABC):

    @property
    @abstractmethod
    def name(self) -> str:
        pass

    @abstractmethod
    def __init__(self, interface: Interface, params=None):
        self.interface = interface
        self.params = merge_params(self.get_default_params(), params)

    # List all available variables
    @staticmethod
    @abstractmethod
    def list_vars() -> List[str]:
        pass

    # List all available observations
    @staticmethod
    @abstractmethod
    def list_obses() -> List[str]:
        pass

    # Get the default params of the environment
    @staticmethod
    @abstractmethod
    def get_default_params() -> dict:
        pass

    # Get current variable
    # Unsafe version (var won't be checked beforehand)
    @abstractmethod
    def _get_var(self, var: str):
        pass

    # Set variable
    # Unsafe version
    @abstractmethod
    def _set_var(self, var: str, x):
        pass

    # Get observation
    # Unsafe version
    @abstractmethod
    def _get_obs(self, obs: str):
        pass

    # Get variable range
    # Unsafe version
    @classmethod
    def _get_vrange(cls, var: str):
        return [0, 1]

    # Safe version of _get_vrange
    @classmethod
    def get_vrange(cls, var: str):
        if var not in cls.list_vars():
            raise Exception(f'Variable {var} doesn\'t exist!')

        return cls._get_vrange(var)

    # Get all the variable ranges
    @classmethod
    def get_vranges(cls, vars=None):
        if vars is None:
            return [cls._get_vrange(var) for var in cls.list_vars()]
        else:
            return [cls.get_vrange(var) for var in vars]

    # Get all the variable ranges
    @classmethod
    def get_vranges_dict(cls, vars=None):
        book = {}
        if vars is None:
            vars = cls.list_vars()
            for var in vars:
                book[var] = cls._get_vrange(var)
        else:
            for var in vars:
                book[var] = cls.get_vrange(var)
        return book

    # Safe version of _get_var
    def get_var(self, var: str):
        if var not in self.list_vars():
            raise Exception(f'Variable {var} doesn\'t exist!')

        return self._get_var(var)

    # Safe version of _set_var
    def set_var(self, var: str, x):
        if var not in self.list_vars():
            raise Exception(f'Variable {var} doesn\'t exist!')

        return self._set_var(var, x)

    # Safe version of _get_obs
    def get_obs(self, obs: str):
        if obs not in self.list_obses():
            raise Exception(f'Observation {obs} doesn\'t exist!')

        return self._get_obs(obs)

    def get_vars(self, vars: List[str]) -> list:
        values = []
        for var in vars:
            values.append(self.get_var(var))

        return values

    def get_vars_dict(self) -> dict:
        vars = self.list_vars()
        book = {}
        for var in vars:
            book[var] = self._get_var(var)

        return book

    def set_vars(self, vars: List[str], values: list):
        assert len(vars) == len(
            values), 'Variables and values number mismatch!'

        for idx, var in enumerate(vars):
            self.set_var(var, values[idx])

    def set_vars_dict(self, book: dict):
        for var, val in book.items():
            self.set_var(var, val)

    def get_obses(self, obses: List[str]) -> list:
        values = []
        for obs in obses:
            values.append(self._get_obs(obs))

        return values

    def get_obses_dict(self) -> dict:
        obses = self.list_obses()
        book = {}
        for obs in obses:
            book[obs] = self.get_obs(obs)

        return book



from PyQt5.QtWidgets import QDialog, QDialogButtonBox, QTextBrowser, QVBoxLayout
from ...utils import ystring


class BadgerReviewDialog(QDialog):
    def __init__(self, parent, routine):
        super().__init__(parent)

        self.routine = routine

        self.init_ui()
        self.config_logic()

    def init_ui(self):
        name = self.routine['name']
        self.setWindowTitle(f'Review routine {name}')

        vbox = QVBoxLayout(self)

        brow_routine = QTextBrowser()
        brow_routine.setText(ystring(self.routine))

        self.btn_ok = btn_ok = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok)

        vbox.addWidget(brow_routine)
        vbox.addWidget(btn_ok)

    def config_logic(self):
        self.btn_ok.accepted.connect(self.accept)



import numpy as np
from PyQt5.QtWidgets import QMessageBox
from PyQt5.QtCore import QObject, pyqtSignal


class BadgerRoutineRunner(QObject):
    finished = pyqtSignal()
    progress = pyqtSignal(list, list)

    def __init__(self, routine, save, verbose=2):
        super().__init__()

        self.routine = routine
        self.save = save
        self.verbose = verbose

    def run(self):
        try:
            self._run()
        except Exception as e:
            print(e)
            # QMessageBox.critical(None, 'Error!', str(e))

        self.finished.emit()

    def _run(self):
        save = self.save
        routine = self.routine
        verbose = self.verbose

        # Save routine if specified
        if save:
            import sqlite3
            from ...db import save_routine

            try:
                save_routine(routine)
            except sqlite3.IntegrityError:
                raise Exception(
                    f'Routine {routine["name"]} already existed in the database! Please choose another name.')

        # Set up and run the optimization
        from ...factory import get_algo, get_intf, get_env
        from ...utils import merge_params, ParetoFront, denorm

        Environment, configs_env = get_env(routine['env'])
        try:
            intf_name = configs_env['interface'][0]
            Interface, _ = get_intf(intf_name)
            intf = Interface()
        except Exception:
            intf = None
        env = Environment(intf, routine['env_params'])

        optimize, configs_algo = get_algo(routine['algo'])
        if not callable(optimize):  # Doing optimization through extensions
            configs = {
                'routine_configs': routine['config'],
                'algo_configs': merge_params(configs_algo, {'params': routine['algo_params']})
            }
            optimize.run(env, configs)
            print('done!')
        else:
            from ...logger import _get_default_logger
            from ...logger.event import Events

            logger = _get_default_logger(verbose)  # log the optimization progress
            var_names = [next(iter(d)) for d in routine['config']['variables']]
            vranges = np.array([d[next(iter(d))]
                            for d in routine['config']['variables']])
            obj_names = [next(iter(d)) for d in routine['config']['objectives']]
            rules = [d[next(iter(d))] for d in routine['config']['objectives']]
            pf = ParetoFront(rules)

            # Make a normalized evaluate function
            def evaluate(X):
                Y = []
                for x in X:
                    _x = denorm(x, vranges[:, 0], vranges[:, 1])
                    env.set_vars(var_names, _x)
                    obses = []
                    obses_raw = []
                    for i, obj_name in enumerate(obj_names):
                        rule = rules[i]
                        obs = float(env.get_obs(obj_name))
                        if rule == 'MAXIMIZE':
                            obses.append(-obs)
                        else:
                            obses.append(obs)
                        obses_raw.append(obs)
                    Y.append(obses)
                    obses_raw = np.array(obses_raw)
                    is_optimal = not pf.is_dominated((_x, obses_raw))
                    solution = (_x, obses_raw, is_optimal, var_names, obj_names)
                    logger.update(Events.OPTIMIZATION_STEP, solution)
                    self.progress.emit(list(_x), list(obses_raw))

                Y = np.array(Y)

                return Y, None, None

            solution = (None, None, None, var_names, obj_names)
            print('')
            logger.update(Events.OPTIMIZATION_START, solution)
            try:
                optimize(evaluate, routine['algo_params'])
            except Exception as e:
                logger.update(Events.OPTIMIZATION_END, solution)
                raise e
            logger.update(Events.OPTIMIZATION_END, solution)



import numpy as np
from abc import ABC, abstractmethod
from typing import List


class Extension(ABC):

    @property
    @abstractmethod
    def name(self) -> str:
        pass

    @abstractmethod
    def __init__(self):
        pass

    # List all available algorithms
    @abstractmethod
    def list_algo(self) -> List[str]:
        pass

    # Get config of an algorithm
    @abstractmethod
    def get_algo_config(self, name: str):
        pass

    # Run an optimization on the given environment
    @abstractmethod
    def run(self, env, configs: dict):
        pass



from PyQt5.QtWidgets import QLineEdit, QListWidget, QListWidgetItem, QWidget, QVBoxLayout, QHBoxLayout
from PyQt5.QtWidgets import QPushButton, QGroupBox, QComboBox, QLineEdit, QPlainTextEdit, QCheckBox
from PyQt5.QtWidgets import QMessageBox
from PyQt5.QtCore import QSize, QThread
from coolname import generate_slug
from ...factory import list_algo, list_env, get_algo, get_env
from ...utils import run_routine, ystring, load_config, config_list_to_dict, normalize_routine
from ..components.variable_item import variable_item
from ..components.objective_item import objective_item
from ..components.routine_runner import BadgerRoutineRunner
from ..windows.review_dialog import BadgerReviewDialog
from ..windows.opt_monitor import BadgerOptMonitor


class BadgerRoutinePage(QWidget):
    def __init__(self, go_home=None):
        super().__init__()

        self.go_home = go_home

        self.algos = list_algo()
        self.envs = list_env()

        self.init_ui()
        self.config_logic()

    def init_ui(self):
        # Set up the layout
        vbox = QVBoxLayout(self)

        # Algo and env configs
        panel_int = QWidget()
        hbox_int = QHBoxLayout(panel_int)
        hbox_int.setContentsMargins(0, 0, 0, 0)

        group_algo = QGroupBox('Algorithm')
        vbox_algo = QVBoxLayout(group_algo)
        self.cb_algo = cb_algo = QComboBox()
        cb_algo.addItems(self.algos)
        cb_algo.setCurrentIndex(-1)
        vbox_algo.addWidget(cb_algo)
        self.edit_algo = edit_algo = QPlainTextEdit()
        edit_algo.setFixedHeight(128)
        vbox_algo.addWidget(edit_algo)
        hbox_int.addWidget(group_algo)

        group_env = QGroupBox('Environment')
        vbox_env = QVBoxLayout(group_env)
        self.cb_env = cb_env = QComboBox()
        cb_env.addItems(self.envs)
        cb_env.setCurrentIndex(-1)
        vbox_env.addWidget(cb_env)
        self.edit_env = edit_env = QPlainTextEdit()
        edit_env.setFixedHeight(128)
        vbox_env.addWidget(edit_env)
        hbox_int.addWidget(group_env)
        vbox.addWidget(panel_int)

        # Configs group
        group_ext = QGroupBox('Configs')
        hbox_ext = QHBoxLayout(group_ext)

        group_var = QGroupBox('Variables')
        vbox_var = QVBoxLayout(group_var)
        action_var = QWidget()
        hbox_action_var = QHBoxLayout(action_var)
        hbox_action_var.setContentsMargins(0, 0, 0, 0)
        self.btn_all_var = btn_all_var = QPushButton('Check All')
        self.btn_un_all_var = btn_un_all_var = QPushButton('Uncheck All')
        self.check_only_var = check_only_var = QCheckBox('Checked Only')
        check_only_var.setChecked(False)
        hbox_action_var.addWidget(btn_all_var)
        hbox_action_var.addWidget(btn_un_all_var)
        hbox_action_var.addStretch()
        hbox_action_var.addWidget(check_only_var)
        vbox_var.addWidget(action_var)
        self.list_var = list_var = QListWidget()
        self.dict_var = {}
        vbox_var.addWidget(list_var)
        hbox_ext.addWidget(group_var)

        group_obj = QGroupBox('Objectives')
        vbox_obj = QVBoxLayout(group_obj)
        action_obj = QWidget()
        hbox_action_obj = QHBoxLayout(action_obj)
        hbox_action_obj.setContentsMargins(0, 0, 0, 0)
        self.btn_all_obj = btn_all_obj = QPushButton('Check All')
        self.btn_un_all_obj = btn_un_all_obj = QPushButton('Uncheck All')
        self.check_only_obj = check_only_obj = QCheckBox('Checked Only')
        check_only_obj.setChecked(False)
        hbox_action_obj.addWidget(btn_all_obj)
        hbox_action_obj.addWidget(btn_un_all_obj)
        hbox_action_obj.addStretch()
        hbox_action_obj.addWidget(check_only_obj)
        vbox_obj.addWidget(action_obj)
        self.list_obj = list_obj = QListWidget()
        self.dict_obj = {}
        vbox_obj.addWidget(list_obj)
        hbox_ext.addWidget(group_obj)

        group_con = QGroupBox('Constraints')
        vbox_con = QVBoxLayout(group_con)
        self.list_con = list_con = QListWidget()
        self.dict_con = {}
        vbox_con.addWidget(list_con)
        hbox_ext.addWidget(group_con)

        vbox.addWidget(group_ext, 1)

        # Action bar
        action_bar = QWidget()
        hbox_action = QHBoxLayout(action_bar)
        hbox_action.setContentsMargins(0, 0, 0, 0)
        self.btn_back = btn_back = QPushButton('Back')
        self.btn_review = btn_review = QPushButton('Review')
        self.btn_run = btn_run = QPushButton('Run Routine')
        btn_run.setFixedWidth(256)
        self.check_save = check_save = QCheckBox('Save as')
        check_save.setChecked(False)
        self.edit_save = edit_save = QLineEdit()
        edit_save.setPlaceholderText(generate_slug(2))
        edit_save.setDisabled(True)
        hbox_action.addWidget(btn_back)
        hbox_action.addWidget(btn_review)
        hbox_action.addStretch(1)
        hbox_action.addWidget(btn_run)
        hbox_action.addWidget(check_save)
        hbox_action.addWidget(edit_save)

        # vbox.addSpacing(16)
        vbox.addWidget(action_bar)

    def config_logic(self):
        self.btn_back.clicked.connect(self.go_home)
        self.cb_algo.currentIndexChanged.connect(self.select_algo)
        self.cb_env.currentIndexChanged.connect(self.select_env)
        self.btn_all_var.clicked.connect(self.check_all_var)
        self.btn_un_all_var.clicked.connect(self.uncheck_all_var)
        self.btn_all_obj.clicked.connect(self.check_all_obj)
        self.btn_un_all_obj.clicked.connect(self.uncheck_all_obj)
        self.check_only_var.stateChanged.connect(self.toggle_check_only_var)
        self.check_only_obj.stateChanged.connect(self.toggle_check_only_obj)
        self.check_save.stateChanged.connect(self.toggle_save)
        self.btn_review.clicked.connect(self.review)
        self.btn_run.clicked.connect(self.run)

    def refresh_ui(self, routine):
        self.algos = list_algo()
        self.envs = list_env()

        if routine is None:
            # Reset the algo and env configs
            self.cb_algo.setCurrentIndex(-1)
            self.cb_env.setCurrentIndex(-1)

            # Reset the routine configs check box status
            self.check_only_var.setChecked(False)
            self.check_only_obj.setChecked(False)

            # Reset the save settings
            name = generate_slug(2)
            self.edit_save.setText('')
            self.edit_save.setPlaceholderText(name)
            self.check_save.setChecked(False)

            return

        # Fill in the algo and env configs
        name_algo = routine['algo']
        idx_algo = self.algos.index(name_algo)
        self.cb_algo.setCurrentIndex(idx_algo)
        self.edit_algo.setPlainText(ystring(routine['algo_params']))

        name_env = routine['env']
        idx_env = self.envs.index(name_env)
        self.cb_env.setCurrentIndex(idx_env)
        self.edit_env.setPlainText(ystring(routine['env_params']))

        # Config the vocs panel
        self.check_only_var.setChecked(True)
        variables = [next(iter(v)) for v in routine['config']['variables']]
        for i in range(self.list_var.count()):
            item = self.list_var.item(i)
            item_widget = self.list_var.itemWidget(item)
            var_name = item_widget.check_name.text()
            try:
                idx = variables.index(var_name)
                vrange = routine['config']['variables'][idx][var_name]
                item_widget.sb_lower.sb.setValue(vrange[0])
                item_widget.sb_upper.sb.setValue(vrange[1])
            except ValueError:
                item_widget.check_name.setChecked(False)

        self.check_only_obj.setChecked(True)
        objectives = [next(iter(v)) for v in routine['config']['objectives']]
        for i in range(self.list_obj.count()):
            item = self.list_obj.item(i)
            item_widget = self.list_obj.itemWidget(item)
            obj_name = item_widget.check_name.text()
            try:
                idx = objectives.index(obj_name)
                rule = routine['config']['objectives'][idx][obj_name]
                idx_rule = 0 if rule == 'MINIMIZE' else 1
                item_widget.cb_rule.setCurrentIndex(idx_rule)
            except ValueError:
                item_widget.check_name.setChecked(False)

        # Config the save settings
        name = routine['name']
        self.edit_save.setPlaceholderText(generate_slug(2))
        self.edit_save.setText(name)
        self.check_save.setChecked(False)

    def select_algo(self, i):
        if i == -1:
            self.edit_algo.setPlainText('')
            return

        name = self.algos[i]
        _, configs = get_algo(name)
        self.edit_algo.setPlainText(ystring(configs['params']))

    def select_env(self, i):
        if i == -1:
            self.edit_env.setPlainText('')
            self.list_var.clear()
            self.dict_var.clear()
            self.list_obj.clear()
            self.dict_obj.clear()
            self.list_con.clear()
            self.dict_con.clear()
            return

        name = self.envs[i]
        _, configs = get_env(name)
        self.edit_env.setPlainText(ystring(configs['params']))

        self.list_var.clear()
        for var in configs['variables']:
            name = next(iter(var))
            vrange = var[name]
            item = QListWidgetItem(self.list_var)
            var_item = variable_item(name, vrange, self.toggle_var)
            item.setSizeHint(var_item.sizeHint())
            self.list_var.addItem(item)
            self.list_var.setItemWidget(item, var_item)
            self.dict_var[name] = item

        self.list_obj.clear()
        for obj in configs['observations']:
            item = QListWidgetItem(self.list_obj)
            obj_item = objective_item(obj, 0, self.toggle_obj)
            item.setSizeHint(obj_item.sizeHint())
            self.list_obj.addItem(item)
            self.list_obj.setItemWidget(item, obj_item)
            self.dict_obj[obj] = item

    def check_all_var(self):
        for i in range(self.list_var.count()):
            item = self.list_var.item(i)
            item_widget = self.list_var.itemWidget(item)
            item_widget.check_name.setChecked(True)

    def uncheck_all_var(self):
        for i in range(self.list_var.count()):
            item = self.list_var.item(i)
            item_widget = self.list_var.itemWidget(item)
            item_widget.check_name.setChecked(False)

    def check_all_obj(self):
        for i in range(self.list_obj.count()):
            item = self.list_obj.item(i)
            item_widget = self.list_obj.itemWidget(item)
            item_widget.check_name.setChecked(True)

    def uncheck_all_obj(self):
        for i in range(self.list_obj.count()):
            item = self.list_obj.item(i)
            item_widget = self.list_obj.itemWidget(item)
            item_widget.check_name.setChecked(False)

    def toggle_check_only_var(self):
        if self.check_only_var.isChecked():
            for i in range(self.list_var.count()):
                item = self.list_var.item(i)
                item_widget = self.list_var.itemWidget(item)
                if not item_widget.check_name.isChecked():
                    item_widget.hide()
                    item.setSizeHint(QSize(0, 0))
        else:
            for i in range(self.list_var.count()):
                item = self.list_var.item(i)
                item_widget = self.list_var.itemWidget(item)
                item_widget.show()
                item.setSizeHint(item_widget.sizeHint())

    def toggle_var(self, name):
        if self.check_only_var.isChecked():
            item = self.dict_var[name]
            item_widget = self.list_var.itemWidget(item)
            if item_widget.check_name.isChecked():
                item_widget.show()
                item.setSizeHint(item_widget.sizeHint())
            else:
                item_widget.hide()
                item.setSizeHint(QSize(0, 0))

    def toggle_check_only_obj(self):
        if self.check_only_obj.isChecked():
            for i in range(self.list_obj.count()):
                item = self.list_obj.item(i)
                item_widget = self.list_obj.itemWidget(item)
                if not item_widget.check_name.isChecked():
                    item_widget.hide()
                    item.setSizeHint(QSize(0, 0))
        else:
            for i in range(self.list_obj.count()):
                item = self.list_obj.item(i)
                item_widget = self.list_obj.itemWidget(item)
                item_widget.show()
                item.setSizeHint(item_widget.sizeHint())

    def toggle_obj(self, name):
        if self.check_only_obj.isChecked():
            item = self.dict_obj[name]
            item_widget = self.list_obj.itemWidget(item)
            if item_widget.check_name.isChecked():
                item_widget.show()
                item.setSizeHint(item_widget.sizeHint())
            else:
                item_widget.hide()
                item.setSizeHint(QSize(0, 0))

    def toggle_save(self):
        if self.check_save.isChecked():
            self.edit_save.setDisabled(False)
        else:
            self.edit_save.setDisabled(True)

    def _compose_routine(self):
        # Compose the routine
        name = self.edit_save.text() or self.edit_save.placeholderText()
        algo = self.cb_algo.currentText()
        env = self.cb_env.currentText()
        algo_params = load_config(self.edit_algo.toPlainText())
        env_params = load_config(self.edit_env.toPlainText())

        variables = []
        for i in range(self.list_var.count()):
            item = self.list_var.item(i)
            item_widget = self.list_var.itemWidget(item)
            if item_widget.check_name.isChecked():
                var_name = item_widget.check_name.text()
                lb = item_widget.sb_lower.sb.value()
                ub = item_widget.sb_upper.sb.value()
                _dict = {}
                _dict[var_name] = [lb, ub]
                variables.append(_dict)

        objectives = []
        for i in range(self.list_obj.count()):
            item = self.list_obj.item(i)
            item_widget = self.list_obj.itemWidget(item)
            if item_widget.check_name.isChecked():
                obj_name = item_widget.check_name.text()
                rule = item_widget.cb_rule.currentText()
                _dict = {}
                _dict[obj_name] = rule
                objectives.append(_dict)

        configs = {
            'variables': variables,
            'objectives': objectives,
            'constraints': None,
        }

        routine = {
            'name': name,
            'algo': algo,
            'env': env,
            'algo_params': algo_params,
            'env_params': env_params,
            'env_vranges': config_list_to_dict(variables),
            'config': configs,
        }

        # Sanity check and config normalization
        try:
            routine = normalize_routine(routine)
        except Exception as e:
            raise e

        return routine

    def review(self):
        try:
            routine = self._compose_routine()
        except Exception as e:
            return QMessageBox.critical(self, 'Error!', str(e))

        dlg = BadgerReviewDialog(self, routine)
        dlg.exec()

    def run_routine(self, routine, save):
        monitor = BadgerOptMonitor(self)
        self.thread_routine = thread_routine = QThread(self)
        self.routine_runner = routine_runner = BadgerRoutineRunner(routine, save)
        routine_runner.moveToThread(thread_routine)
        thread_routine.started.connect(routine_runner.run)
        routine_runner.finished.connect(thread_routine.quit)
        routine_runner.finished.connect(routine_runner.deleteLater)
        thread_routine.finished.connect(routine_runner.deleteLater)
        routine_runner.progress.connect(monitor.update)
        thread_routine.start()
        monitor.exec()

    def run(self):
        try:
            routine = self._compose_routine()
        except Exception as e:
            return QMessageBox.critical(self, 'Error!', str(e))

        save = self.check_save.isChecked()

        try:
            self.run_routine(routine, save)
        except Exception as e:
            QMessageBox.critical(self, 'Error!', str(e))



# Borrowed from bayes_opt
class Colours:
    """Print in nice colours."""

    BLUE = '\033[94m'
    BOLD = '\033[1m'
    CYAN = '\033[96m'
    DARKCYAN = '\033[36m'
    END = '\033[0m'
    GREEN = '\033[92m'
    PURPLE = '\033[95m'
    RED = '\033[91m'
    UNDERLINE = '\033[4m'
    YELLOW = '\033[93m'

    @classmethod
    def _wrap_colour(cls, s, colour):
        return colour + s + cls.END

    @classmethod
    def black(cls, s):
        """Wrap text in black."""
        return cls._wrap_colour(s, cls.END)

    @classmethod
    def blue(cls, s):
        """Wrap text in blue."""
        return cls._wrap_colour(s, cls.BLUE)

    @classmethod
    def bold(cls, s):
        """Wrap text in bold."""
        return cls._wrap_colour(s, cls.BOLD)

    @classmethod
    def cyan(cls, s):
        """Wrap text in cyan."""
        return cls._wrap_colour(s, cls.CYAN)

    @classmethod
    def darkcyan(cls, s):
        """Wrap text in darkcyan."""
        return cls._wrap_colour(s, cls.DARKCYAN)

    @classmethod
    def green(cls, s):
        """Wrap text in green."""
        return cls._wrap_colour(s, cls.GREEN)

    @classmethod
    def purple(cls, s):
        """Wrap text in purple."""
        return cls._wrap_colour(s, cls.PURPLE)

    @classmethod
    def red(cls, s):
        """Wrap text in red."""
        return cls._wrap_colour(s, cls.RED)

    @classmethod
    def underline(cls, s):
        """Wrap text in underline."""
        return cls._wrap_colour(s, cls.UNDERLINE)

    @classmethod
    def yellow(cls, s):
        """Wrap text in yellow."""
        return cls._wrap_colour(s, cls.YELLOW)



from PyQt5.QtWidgets import QApplication
from PyQt5 import QtCore
import sys
import ctypes
from .windows.main_window import BadgerMainWindow

# Fix the scaling issue on multiple monitors w/ different scaling settings
if sys.platform == 'win32':
    ctypes.windll.shcore.SetProcessDpiAwareness(1)

if hasattr(QtCore.Qt, 'AA_EnableHighDpiScaling'):
    QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)

if hasattr(QtCore.Qt, 'AA_UseHighDpiPixmaps'):
    QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)

# if hasattr(QtCore.Qt, 'HighDpiScaleFactorRoundingPolicy'):
#     QApplication.setAttribute(
#         QtCore.Qt.HighDpiScaleFactorRoundingPolicy.PassThrough)


def launch_gui():
    app = QApplication(sys.argv)

    window = BadgerMainWindow()

    window.show()
    sys.exit(app.exec())



import numpy as np
from PyQt5.QtWidgets import QDialog, QDialogButtonBox, QVBoxLayout
from PyQt5 import QtCore
import pyqtgraph as pg


class BadgerOptMonitor(QDialog):
    def __init__(self, parent):
        super().__init__(parent)

        self.init_ui()
        self.config_logic()

    def init_ui(self):
        self.setWindowTitle('Opt Monitor')
        self.resize(1280, 640)

        vbox = QVBoxLayout(self)

        monitor = pg.GraphicsLayoutWidget()
        # monitor.resize(1000, 600)
        pg.setConfigOptions(antialias=True)

        self.plot_obj = plot_obj = monitor.addPlot(title='Evaluation History (Y)')
        plot_obj.setLabel('left', 'objectives')
        plot_obj.setLabel('bottom', 'iterations')
        plot_obj.showGrid(x=True, y=True)

        self.plot_var = plot_var = monitor.addPlot(title='Evaluation History (X)')
        plot_var.setLabel('left', 'variables')
        plot_var.setLabel('bottom', 'iterations')
        plot_var.showGrid(x=True, y=True)

        self.btn_ok = btn_ok = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok)

        vbox.addWidget(monitor)
        vbox.addWidget(btn_ok)

    def config_logic(self):
        self.colors = ['b', 'g', 'r', 'c', 'm', 'y', 'w']
        self.symbols = ['o', 't', 't1', 's', 'p', 'h', 'd']
        self.vars = []
        self.objs = []
        self.curves_var = []
        self.curves_obj = []

        self.btn_ok.accepted.connect(self.accept)

    def update(self, vars, objs):
        self.vars.append(vars)
        self.objs.append(objs)

        if not self.curves_obj:
            for i in range(len(objs)):
                color = self.colors[i % 7]
                symbol = self.symbols[i % 7]
                _curve = self.plot_obj.plot(pen=color, symbol=symbol)
                self.curves_obj.append(_curve)

        if not self.curves_var:
            for i in range(len(vars)):
                color = self.colors[i % 7]
                symbol = self.symbols[i % 7]
                _curve = self.plot_var.plot(pen=color, symbol=symbol)
                self.curves_var.append(_curve)

        for i in range(len(objs)):
            self.curves_obj[i].setData(np.array(self.objs)[:, i])

        for i in range(len(vars)):
            self.curves_var[i].setData(np.array(self.vars)[:, i])



from setuptools import setup, find_packages

setup(
    name='badger-opt',
    version='0.4.2',
    description='Core of the Badger optimizer',
    url='https://github.com/SLAC-ML/Badger',
    author='Zhe Zhang',
    author_email='zhezhang@slac.stanford.edu',
    license='GPL',
    package_dir={'': 'src'},
    packages=find_packages(where='src'),
    install_requires=[
        'numpy',
        'pyyaml',
        'coolname',
        'pyqt5',
        'pyqtgraph'
    ],
    python_requires='>=3.6',
    entry_points={
        'console_scripts': [
            'badger = badger.__main__:main'
        ]
    },
)



from pkg_resources import get_distribution
from ..factory import BADGER_PLUGIN_ROOT, BADGER_EXTENSIONS
from ..db import BADGER_DB_ROOT
from ..utils import yprint


def show_info(args):
    if args.gui:
        from ..gui import launch_gui

        launch_gui()
    else:
        info = {
            'name': 'Badger the optimizer',
            'version': get_distribution('badger-opt').version,
            'plugin root': BADGER_PLUGIN_ROOT,
            'database root': BADGER_DB_ROOT,
        }

        extensions = list(BADGER_EXTENSIONS.keys())
        if extensions:
            info['extensions'] = extensions

        yprint(info)
        # print(f'Badger the optimizer')
        # print('====================')
        # print(f'version      : {version}')
        # print(f'plugins root : {BADGER_PLUGIN_ROOT}')
